.nr Cl 3
.nr Hb 3
.nr Hs 3
.nr Ff 0
.COVER
.TL
Application Programming Using the GNOME Libraries
.AU "George Lebl <jirka@5z.com>"
.AF "The GNOME Project"
.AS
In this tutorial, you will receive an overview of the GNOME libraries.
You will learn how to speed up development of applications by using the
many utility routines and objects available through the GNOME libraries,
and how to make the GUI more consistent by using standard GNOME UI
components. Focus will also be given to C applications using the GTK+
toolkit.
.AE
.COVEND

.HU "Credits, Copyrights and Other Such Informations"
.P
All of the code given in this tutorial is under the GNU General
Public License or the GNU Library General Public License. It
was written by me or the rest of the Gnome core team.
.P
Also I would like to apologize if some of my English is not
correct, as English is not my first language. I hope that
my C is better then my English.

.H 1 "GNOME Libraries Overview"
.P
First we are going to look at the structure of the GNOME libraries to
see what they can offer. Here is a listing of the different libraries
that are present in the gnome-libs package:
.BL
.LI
.B libgnome " - Toolkit independent utility library"
.LI
.B libgnomeui " - Toolkit dependent library"
.LI
.B libgnorba " - Library for using ORBit corba implementation with gnome"
.LI
.B gtk-xmhtml " - xmHTML widget ported to gtk, used in the help browser"
.LI
.B zvt " - A very lean and mean terminal emulator widget"
.LI
.B libvfs " - A virtual file-system library used in Midnight Commander"
.LI
.B libart_lgpl " - A library used for nice anti-aliased graphics"
.LE
.P
We will not cover gtk-xmhtml,
zvt, libvfs, libart_lgpl and libgnorba as they are mostly specialty
libraries and we won't have the time.
.P
We can see a clear division between the \fBlibgnome\fR and \fBlibgnomeui\fR
libraries. The former is used in a toolkit independent fashion and could
even be used for command line programs that never use X. The latter is
the library which supplies the standard widgets and an application framework
for applications written using \fBGTK+\fR. It is conceivable to write
applications with other toolkits, but nobody as of yet has written a
\fBlibgnomeui\fR with a different toolkit, and I doubt it will happen
soon, as GTK+ is a really great toolkit.

.H 1 "GTK+ Programming"
.H 2 "Overview"
.P
GTK+ is a C based toolkit for programming graphical applications in X
windows. It is highly object oriented and has bindings to many popular
languages, such as C++, Objective C, Perl, TOM, Guile, Python, etc ...
GTK+ also uses GLib, which is a very useful C library, and includes
things to help porting to different architectures, and containers
such as a linked list or a hash.  If you are already familiar with GTK+,
you are now free to get bored.

.H 2 "GLib"
.H 3 "Naming Conventions"
.P
GLib is a utility library which is heavily used in GTK+ and most of GNOME.
GLib's functions are named starting with \fBg_\fR (such as \fBg_strdup\fR),
GLib's typedefs for common types are just prefixed with a \fBg\fR (such as
\fBgint32\fR), and GLib's structures are capitalized and start with \fBG\fR
(such as \fBGHashTable\fR).

.H 3 "Typedefs"
.P
GLib provides some portability typedefs, such as \fBgint32\fR, \fBgint8\fR, or
just simplification typedefs such as \fBguint\fR (for \fBunsigned int\fR) and
some typedefs which are present just to keep everything looking consistent,
such as \fBgint\fR which just typedefs to \fBint\fR, or \fBgpointer\fR which
is a typedef of \fBvoid *\fR.

.H 3 "Portability and Utility Functions".
.P
There are some functions that have different implementations across
different systems or are not extremely safe, or don't exist at all on some
systems, so GLib provides it's own implementations or wrappers that have a
constant behavior and usually check their arguments. Examples of such
functions are \fBg_strdup\fR, \fBg_snprintf\fR or \fBg_malloc\fR and
\fBg_free\fR and more.
.P
GLib also includes a lot of utility functions such as \fBg_strconcat\fR or
\fBg_strdup_printf\fR and others, that are just plain useful.

.H 3 "Containers"
.P
Although probably the best part of GLib are the containers. Such as linked
lists (\fBGList\fR) or re-sizable arrays (\fBArray\fR) or hash tables
(\fBGHashTable\fR) and others. The easiest to use are \fBGList\fR's.
The basic \fBGList\fR structure is just a single node of the linked list
and you can put your data into the data pointer in the \fBGList\fR
structure. To store a linked list you just store a pointer to the
first node of the list. So code that would create a linked list of
two elements which are strdup'ed strings, and later free that list
and the strings would look like:
.VERBON 16
GList *list = NULL; /*the actual list pointer*/
GList *li; /*just a temporary pointer to a node used for iterating
	     over the list*/
\&...
/*here we add two strings to the list*/
list = g_list_append(list,g_strdup("String 1"));
list = g_list_append(list,g_strdup("String 2"));
\&...
/*here we loop though the list, freeing all the strings and then
  we free the list itself*/
for(li = list; li!= NULL; li = g_list_next(li)) {
	char *string = li->data;
	g_free(string);
}
g_list_free(list);
.VERBOFF
.P
For more information look at the \fBglib.h\fR header file and at the
documentation on the \fBwww.gtk.org\fR web site.

.H 2 "GTK+"
.H 3 "GUI Basics"
.P
Writing a GTK+ based GUI application is in essence extremely simple, and
we'll use the Hello World example from Ian Main's excellent GTK+ tutorial,
which is a very useful guide to writing gnome applications. But first we'll
talk about the basic philosophy behind GTK+.
.P
GTK+ is a container based toolkit, meaning you don't specify where the
widget is, but you specify in what container it is. Some widgets,
such as a window or a frame or a button, are containers that hold only
one other widget. For example a button with a label is actually a button
into which we added a label widget. If you need to put more widgets into
that container, you will need to add another container into them, one
that holds more then one widget such as a horizontal box.
.P
In fact most layout of windows is usually done with containers such as
horizontal boxes, vertical boxes and tables, those are the most important
to learn. A horizontal box is a widget that you can add several widgets
into and they will be added in a horizontal row. The height of the
horizontal box is the height of the highest widget added, and the
length is the length of all widgets combined. Vertical box behaves exactly
the same, except that it's vertical instead of horizontal. A table
can take in widgets at different rows and columns.

.DS
.PS < figure1.pic
.FG "Example window hierarchy"
.DE

.H 3 "GTK+ Object Model"
.P
Gtk's object model is an object oriented framework for C. It includes
singular object inheritance, virtual methods, signals, runtime object
modification, runtime type checking, and other goodies. While writing
a GTK+ object is more involved then say writing an object in something
like Java, it does have many advantages. GTK+ is an object model which
doesn't require inheritance for most things you do with objects. For one,
since methods are just functions that take the pointer to the object
as the first argument, it's easy to write more methods in your own code,
which are missing in the original object.

.H 3 "Data on Objects"
.P
There is a way to store
arbitrary named data in objects to extend the object. This is done with
the method, \fBgtk_object_set_data\fR (or \fBgtk_object_set_user_data\fR
for a single unnamed pointer). To retrieve data, one uses
\fBgtk_object_get_data\fR. Example:
.VERBON 16
GtkObject *obj;
void *some_pointer;
\&...
/*here we set "some_data" data on obj to point to some_pointer*/
gtk_object_set_data(obj,"some_data",some_pointer);
\&...
/*retrieve pointer to some_data from obj and store it in
  some_pointer*/
some_pointer = gtk_object_get_data(obj,"some_data");
.VERBOFF
The pointer can be a pointer to anything since it's manipulated as a (void *).

.H 3 "GTK+/GNOME Naming Conventions"
.P
Both GTK+ and GNOME use the same naming convention when naming objects
and functions. GTK+ uses a prefix of \fBgtk_\fR for functions, and \fBGtk\fR
for objects, and GNOME uses \fBgnome_\fR and \fBGnome\fR. When a function
is a method for an object, the name (lower case) is appended to the prefix.
For example the button object is named \fBGtkButton\fR (that is the name
of the C struct holding the data for the object), and say the "\fBnew\fR"
method for \fBGtkButton\fR is then called \fBgtk_button_new\fR. Macros
associated with objects use the same naming convention as functions, but
are all capitalized. For example a macro that casts an object to a
\fBGtkButton\fR is called \fBGTK_BUTTON\fR. There are exceptions, notably
the type checking macro, which is called \fBGTK_IS_BUTTON\fR for
\fBGtkButton\fR.

.H 3 "Using GTK+ Methods"
.P
Since GTK+ is object oriented, it uses inheritance for it's widgets. For
example \fBGtkHBox\fR and \fBGtkVBox\fR are derived from \fBGtkBox\fR.
And thus you can use any \fBGtkBox\fR method on a \fBGtkVBox\fR or
\fBGtkHBox\fR. However you need to cast the \fBGtkVBox\fR object
to \fBGtkBox\fR before you call the function. This could be done with
standard C casts such as:
.VERBON 16
GtkVBox *vbox;
\&...
gtk_box_pack_start((GtkBox *)vbox, ...);
\&...
.VERBOFF
This would work, however it is unsafe. GTK+ provides a mechanism of checking
the types, so that it can warn you if you are casting an object which
does not derive from the object you are casting to, or if you try to
cast a NULL pointer. The macro is all capital name of the widget. For example
the above code snippet would be
.VERBON 16
GtkVBox *vbox;
\&...
gtk_box_pack_start(GTK_BOX(vbox), ...);
\&...
.VERBOFF
.P
GNOME uses the exact same form so anything you learn about GTK+ can be used
for GNOME widgets, you just replace the GTK prefix with GNOME.

.H 3 "Example Hello World Program"
.P
Here is the promised example code for the hello world program.
It doesn't use any advanced containers, just a window and a button
onto which a label is added. It illustrates the basic workings
of a GUI program written in GTK+. Don't be scared by it's size,
it's mostly comments.
.VERBON 16
/* example-start helloworld helloworld.c */

#include <gtk/gtk.h>

/* this is a callback function. the data arguments are ignored in
 * this example.. More on callbacks below. */
void
hello (GtkWidget *widget, gpointer data)
{
	g_print ("Hello World\\n");
}

gint
delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	g_print ("delete event occurred\\n");
	/* if you return FALSE in the "delete_event" signal
	 * handler, GTK will emit the "destroy" signal.
	 * Returning TRUE means you don't want the window
	 * to be destroyed. This is useful for popping up
	 * 'are you sure you want to quit ?' type dialogs. */

	/* Change TRUE to FALSE and the main window will
	 * be destroyed with a "delete_event". */

	return (TRUE);
}

/* another callback */
void
destroy (GtkWidget *widget, gpointer data)
{
	gtk_main_quit ();
}

int
main (int argc, char *argv[])
{
	/* GtkWidget is the storage type for widgets */
	GtkWidget *window;
	GtkWidget *button;

	/* this is called in all GTK applications.
	 * arguments are parsed from the command line and
	 * are returned to the application. */
	gtk_init (&argc, &argv);

	/* create a new window */
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

	/* when the window is given the "delete_event" signal
	 * (this is given by the window manager, usually by
	 * the 'close' option, or on the titlebar), we ask
	 * it to call the delete_event () function as defined
	 * above.  The data passed to the callback function
	 * is NULL and is ignored in the callback function. */
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
			    GTK_SIGNAL_FUNC (delete_event), NULL);

	/* here we connect the "destroy" event to a signal
	 * handler. This event occurs when we call
	 * gtk_widget_destroy() on the window, or if we
	 * return 'FALSE' in the "delete_event" callback. */
	gtk_signal_connect (GTK_OBJECT (window), "destroy",
			    GTK_SIGNAL_FUNC (destroy), NULL);

	/* sets the border width of the window. */
	gtk_container_border_width (GTK_CONTAINER (window), 10);

	/* creates a new button with the label "Hello World". */
	button = gtk_button_new_with_label ("Hello World");

	/* When the button receives the "clicked" signal, it
	 * will call the function hello() passing it NULL as
	 * it's argument.  The hello() function is defined
	 * above. */
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    GTK_SIGNAL_FUNC (hello), NULL);

	/* This will cause the window to be destroyed by
	 * calling gtk_widget_destroy(window) when "clicked".
	 * Again, the destroy signal could come from here,
	 * or the window manager. */
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
				   GTK_OBJECT (window));

	/* this packs the button into the window
	 * (a gtk container). */
	gtk_container_add (GTK_CONTAINER (window), button);

	/* the final step is to display this newly created
	 * widget... */
	gtk_widget_show (button);

	/* and the window */
	gtk_widget_show (window);

	/* all GTK applications must have a gtk_main().
	 * Control ends here and waits for an event to occur
	 * (like a key press or mouse event). */
	gtk_main ();

	return 0;
}
/* example-end */
.VERBOFF
.P
For more information look at the header files in <prefix>/include/gtk/ and
<prefix>/include/gdk/ and at the documentation on the \fBwww.gtk.org\fR
web site.

.H 1 "GNOME Programming"
.H 2 "Introduction"
.H 3 "What Is a GNOME Program"
.P
A GNOME program is a GTK+ GUI application, which makes use of the
GNOME libraries. The GNOME libraries make it possible to have similar
look and feel among applications, and to make simple things, simple
to program. Plus the GNOME libraries add a whole bunch of widgets that
simply don't fit into GTK+.

.H 3 "Very Basic GNOME Program"
.P
The following program creates a basic gnome window and adds a horizontal
box into which it packs two buttons, which (when pressed) print a string
onto the stdout of the terminal you started the application from.
.VERBON 16
/*
 * A simple Gnome program, outside of GNOME tree, not using i18n
 * buttons.c
 */
/* the very basic gnome include */
#include <gnome.h>

/* a callback for the buttons */
static void
button_clicked(GtkWidget *button, gpointer data)
{
	/* the string to print is passed though the data field
	   (which is a void *) */
	char *string = data;
	/* print a string on the standard output */
	g_print(string);
}

int
main(int argc, char *argv[])
{
	GtkWidget *app;
	GtkWidget *button;
	GtkWidget *hbox;

	/* Initialize GNOME, this is very similar to gtk_init */
	gnome_init ("buttons-basic-example", "0.1", argc, argv);
	
	/* Create a Gnome app widget, which sets up a basic window
	   for your application */
	app = gnome_app_new ("buttons-basic-example",
			     "Basic GNOME Application");

	/* bind "delete_event", which is the event we get when
	   the user closes the window with the window manager,
	   to gtk_main_quit, which is a function that causes
	   the gtk_main loop to exit, and consequently to quit
	   the application */
	gtk_signal_connect (GTK_OBJECT (app), "delete_event",
			    GTK_SIGNAL_FUNC (gtk_main_quit),
			    NULL);

	/* create a horizontal box for the buttons and add it
	   into the app widget */
	hbox = gtk_hbox_new (FALSE,5);
	gnome_app_set_contents (GNOME_APP (app), hbox);

	/* make a button and add it into the horizontal box,
	   and bind the clicked event to call button_clicked */
	button = gtk_button_new_with_label("Button 1");
	gtk_box_pack_start (GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    GTK_SIGNAL_FUNC (button_clicked),
			    "Button 1\\n");

	/* and another button */
	button = gtk_button_new_with_label("Button 2");
	gtk_box_pack_start (GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			    GTK_SIGNAL_FUNC (button_clicked),
			    "Button 2\\n");
	
	/* show everything inside this app widget and the app
	   widget itself */
	gtk_widget_show_all(app);
	
	/* enter the main loop */
	gtk_main ();
	
	return 0;
}
.VERBOFF
.P
Please note the use of gnome_init instead of gtk_init, and GnomeApp widget
instead of just a regular GtkWindow. We will go into detail of these later.

.H 2 "Overview"
.P
OK now we look at what the different libraries that we are going to cover
do. First we look at the \fBlibgnome\fR library and it's functionality,
then we'll cover \fBlibgnomeui\fR to complete all the basic functionality
of a gnome program. We will take a look at the \fBgnome-canvas\fR in detail,
as it is extremely powerful and useful widget. We'll also say a few
things about drag and drop.

.H 2 "Using the libgnome Library"
.P
The \fBlibgnome\fR library is the non-toolkit specific utility library
for GNOME applications, and includes things like configuration file
reading, .desktop file handling, special GLib like utility routines,
getting the standard file locations for GNOME, handling mime types,
handling meta-data on files, sound, "triggers", and other useful things
one could want to use. Also say that you are writing an application
in say motif, but you want your app to be more GNOME friendly. Then you
could use this library to make your application work well with other
GNOME programs.

.H 3 "Configuration Files"
.P
The \fBgnome-config\fR routines provide an easy way to store configuration
info in files. To see a full list of the routines, look in the
\fBlibgnome/gnome-config.h\fR header file.
.P
The routines all working with a path. The path is a Unix like path, but the
root is set to the \fI~/.gnome/\fR directory. So
\fI/some/config/path/file/sectionA/keyB\fR, refers to the file
\fI~/.gnome/some/config/path/file\fR, and inside the file using section
\fIsectionA\fR and key \fIkeyB\fR.

.H 4 "Reading Configuration Info"
.P
To read configuration information \fBgnome_config_get_*\fR functions are
used. the \fB*\fR is replaced by the type of the data, it can be \fBint\fR,
\fBfloat\fR, \fBstring\fR, \fBbool\fR and \fBvector\fR.
The \fBint\fR functions work with \fBgint\fR, \fBfloat\fR functions work
with \fBgdouble\fR, \fBstring\fR functions work with \fBgchar *\fR, \fBbool\fR
functions work with \fBgboolean\fR and \fBvector\fR work with an argc like
array of strings (\fBgint\fR and \fBgchar **\fR). For the
\fBgnome_config_get_*\fR functions, the default to be returned if the
file section or key are not found can be appended to the path after an
equals sign. If you need to know if the default was used, you can
append \fB_with_default\fR to the function name and add a parameter
which is a \fBgboolean *\fR, though which the function returns whether
it used the default or if it actually found a real value. Example follows:
.VERBON 16
int counter;
char *text;
gboolean def;
\&...
counter = gnome_config_get_int_with_default("/example/counter=1",
					    &def);
if(def) g_print("Default used for counter!\\n");
text = gnome_config_get_string("/example/text=TEXT");
\&...
g_free(text);
.VERBOFF
Note that the string returned by \fBgnome_config_get_string\fR should be
freed with \fBg_free\fR, the vector from \fBgnome_config_get_vector\fR
should also be freed with \fBg_free\fR.

.H 4 "Writing Configuration Info"
.P
To write configuration info to files, the \fBgnome_config_set_*\fR functions
are used. The use is very similar to above to the \fBgnome_config_get_*\fR
functions. The types used are exactly the same. Except with the "set"
functions, you pass the data you want to store after the path, and there is
no default inside the path. If the directory in the path doesn't exist
it will get created when the functions are written to disk. After you
set all your data, you need to call \fBgnome_config_sync\fR to actually
write your data to file. The library will not write the data to file
immediately for efficiency reasons. Example follows:
.VERBON 16
char *text;
int counter;
\&...
/*after we have set text and counter to some values we can
  write them to our config location*/
gnome_config_set_int("/example/counter",counter);
gnome_config_set_string("/example/text",text);
gnome_config_sync();
.VERBOFF

.H 4 "Privacy Functions"
.P
If you want to store sensitive data, that other users should not read,
use the \fBgnome_config_private_*\fR functions, which have exactly the
same behavior as the above functions, with the exception of
\fBgnome_config_sync\fR (and a few others) which doesn't have a private
equivalent since it works for both. The difference is that these functions
write to a directory called \fI~/.gnome_private\fR on which 0700 permissions
are enforced. This is not extremely secure, but because of the highly
brain-dead US export restrictions, we can't really use encryption.

.H 4 "Using gnome-config for Arbitrary Files"
.P
If you wish to use \fBgnome-config\fR for reading and writing of arbitrary
files on the file-system (as long as those files are in the \fBgnome-config\fR
format), you can just prepend \fB'='\fR to the beginning of the path and
another \fB'='\fR to the end of the file name. Example follows:
.VERBON 16
char buf[256];
\&...
/*write some bogus data to a temporary file*/
g_snprintf(buf,256,"=%s=/section/key",tmpnam(tmpnam));
gnome_config_set_int(buf,999);
gnome_config_sync();
.VERBOFF
Note that it doesn't really make sense to use the private versions when
using an arbitrary absolute path, as there will be absolutely no difference
between the two.

.H 4 "Automatic Prefixes"
.P
Sometime, especially if you have a long path, would be much easier
to say have the config automatically prefix the path with a given string.
This is what \fBgnome_config_push_prefix\fR and \fBgnome_config_pop_prefix\fR
are for. You pass the string you want to prefix to
\fBgnome_config_push_prefix\fR and call \fBgnome_config_pop_prefix\fR when
you are done. Note that these functions are common between private and
normal config functions. Example:
.VERBON 16
gnome_config_push_prefix("/file/section");
gnome_config_set_int("key1",1);
gnome_config_set_int("key2",2);
gnome_config_set_int("key3",-88);
gnome_config_pop_prefix();
.VERBOFF

.H 4 "Misc gnome-config Stuff"
.P
If you need to remove a file in your configuration file, you will use
\fBgnome_config_clean_file\fR. This function will schedule that file
to be deleted on the next \fBgnome_config_sync\fR. You can do a 
\fBgnome_config_clean_file\fR and then use the file and then do
\fBgnome_config_sync\fR, and it will have the expected behavior.
.P
If you have written to a file or read from a file and want \fBgnome-config\fR
to drop it from memory, use \fBgnome_config_drop_file\fR. This is used
if you want to forget changes done to that file, or to simply conserve
memory, since \fBgnome-config\fR will otherwise keep a copy of the
data in memory for faster access.

.H 3 "\&.desktop Files"
.P
The \fB.desktop\fR files are the files that contain information about
programs. The files are in the \fBgnome-config\fR format and are internally
read using \fBgnome-config\fR. Your app definitely needs one of these
files installed in the system menu paths if it wants to be added to the menu.
.P
You can use \fBgnome_desktop_entry_*\fR functions to manipulate these files.
These functions work with a structure called \fBGnomeDesktopEntry\fR and
you should look at the \fBlibgnome/gnome-dentry.h\fR header file for the format
of this structure.
.P
The basic functions that you use to manipulate these files are
\fBgnome_desktop_entry_load\fR which returns a newly allocated
\fBGnomeDesktopEntry\fR structure, \fBgnome_desktop_entry_launch\fR
which takes the \fBGnomeDesktopEntry\fR structure as an argument and
launches the program it describes and \fBgnome_desktop_entry_free\fR
which frees the allocated memory with the structure.
.P
An example .desktop file for your app might look like:
.VERBON 16
[Desktop Entry]
Name=Clock
Name[cz]=Hodiny
Comment=Digital Clock
Comment[cz]=Digitalni Hodiny
Exec=digital-clock
Icon=clock.png
Terminal=0
Type=Application
.VERBOFF
You will notice that there are translations for Name and Comment
fields in Czech. For gnome programs to notice your .desktop file,
it is usually located somewhere under \fI<prefix>/share/apps/\fR, which
contains the hierarchy of the system menus. For the system to find
your icon, your icon should be placed inside the
\fI<prefix>/share/pixmaps\fR directory. Note that the prefix refers
to the location where GNOME was installed.

.H 3 "Utility and Files"
.H 4 "Files"
.P
There is a standard way to find files that belong to gnome installation,
you shouldn't really be using your own logic to find them and you should
use these functions to get filenames for icons, sounds or other data.
The functions are:
.VERBON 16
char *gnome_libdir_file (const char *filename); 
char *gnome_datadir_file (const char *filename);
char *gnome_sound_file (const char *filename); 
char *gnome_pixmap_file (const char *filename); 
char *gnome_config_file (const char *filename);
char *gnome_unconditional_libdir_file (const char *filename); 
char *gnome_unconditional_datadir_file (const char *filename); 
char *gnome_unconditional_sound_file (const char *filename); 
char *gnome_unconditional_pixmap_file (const char *filename); 
char *gnome_unconditional_config_file (const char *filename);
.VERBOFF
These functions return a newly \fBg_malloc\fRed string and you should
use \fBg_free\fR on the string when you are done. The
\fBgnome_unconditional_*\fR functions don't check if the file actually
exist and will just return a file name. The normal functions will check
and return \fBNULL\fR if the file doesn't exist. So you shouldn't use
those functions when you will do saving. As an example we want to
get a pixmap from the standard pixmap directory, for example we need
to get the "gnome-help.png" icon:
.VERBON 16
char *name;
\&...
name = gnome_pixmap_file("gnome-help.png");
if(!name) {
	g_warning("gnome-help.png doesn't exist!");
} else {
	/*here we use the file*/
	\&...
	g_free(name);
}
.VERBOFF
.P
Also of interest are the functions (actually macros)
\fBgnome_util_home_file\fR and \fBgnome_util_user_home\fR.
\fBgnome_util_home_file\fR takes one argument (string) and
returns a newly allocated string with the home directory and .gnome
prepended to the file. So for example if you pass it say
\fIsomeconfigfile\fR, it would return \fI/home/jirka/.gnome/someconfigfile\fR.
Similar is the \fBgnome_util_user_home\fR, it takes one argument and
returns the file with just the home directory added. So if you pass it
\&\fI.dotfile\fR, it would return \fI/home/jirka/.dotfile\fR.

.H 4 "Utility"
.P
There are also a number of GLib like named functions to make your life
easier, of note would be \fBg_file_exists\fR which takes a filename and
returns \fBTRUE\fR if it exists or \fBFALSE\fR if it doesn't, or
\fBg_concat_dir_and_file\fR which takes a directory name and a file
name, and takes care of the \fB'/'\fR issue, this is useful when
working with strings where you don't want to check for the '/', you
just want to append a directory to some file, or another directory. Note
that you should \fBg_free\fR the string you get as usual. For more utility
functions, look into \fBlibgnome/gnome-util.h\fR, it is well commented.

.H 3 "Mime Types"
.P
Sometimes it's useful to know the mime-type of a file. You can do this
by using the \fBgnome_mime_type_or_default\fR function, which takes
two arguments, the filename and a default mime-type string which it
will return if it can't figure out the mime type from the filename.
This call doesn't actually look into the file, it tries to guess the
type by looking at the filename itself. Also the string it returns
is a pointer to it's internal database and you should not free it
as that would likely result in a segfault later on. You can also
use \fBgnome_mime_type\fR which will return NULL if it
can't guess the mime-type.
.P
It is also possible to work with URI lists, such as the ones used sometimes
in Drag and Drop. Usually from an URI list you want to extract a list of
filenames that you just received. For that you use the
\fBgnome_uri_list_extract_filenames\fR function, which takes the URI
list as a string argument, and returns a \fBGList *\fR of newly allocated
strings. Once you are done with the files, you should free the strings
and the list. You can use the utility routine \fBgnome_uri_list_free_strings\fR
to do this for you.
.P
In the following example I write a drag and drop handler
that takes the files and finds out their mime information, then you
could just write code that can do things based on the mime type
of the files.
.VERBON 16
/*this is the handler for the drag_data_receive signal, assuming our
  widget only accepts the "text/uri-list" mime type of data, drag and
  drop is a more complicated topic and you should read up on GTK+
  documentation for better treatment*/
static void
dnd_drop_internal (GtkWidget        *widget,
		   GdkDragContext   *context,
		   gint              x,
		   gint              y,
		   GtkSelectionData *selection_data,
		   guint             info,
		   guint             time)
{
	GList *files, *li;

	/*here we extract the filenames from the URI-list we received*/
	files = gnome_uri_list_extract_filenames(selection_data->data);

	/*here we loop though the files and get their mime-type*/
	for(li = files; li!=NULL ; li = g_list_next(li)) {
		  char *mimetype;
		  char *filename = li->data;

		  /*guess the mime type of the file*/
		  mimetype = gnome_mime_type(filename);

		  /*if we can't guess it, just loop to the
		    next filename*/
		  if(!mimetype) continue;

		  /*here comes code that can actually do something
		    based on the mime-type of the file we received*/
		  \&...
	}
	/*free the list of files we got*/
	gnome_uri_list_free_strings (files);
}
.VERBOFF
Note how easy it is to find out what files you got, and what type
they are. You would just need to add some code instead of the three
dots that actually compares the mime strings you got to some you have
to figure out what you can do with the files.

.H 3 "Meta Data"
.P
Sometimes it is useful to store some information along with a filename,
this can be done easily with the \fBgnome-metadata\fR. It is a set
of functions to manage this data. Since Unix doesn't natively
support meta-data, you have to help it yourself. For example
if your app copies, renames or deletes files, use the following
functions.
.VERBON 16
int gnome_metadata_rename (const char *from, const char *to);
int gnome_metadata_copy (const char *from, const char *to);
int gnome_metadata_delete (const char *file);
.VERBOFF
These functions don't actually do the operations on the files, they just
change the meta-data accordingly. So if your app does any of these
operations, it is nicer towards other apps, that it notifies the
meta-data database of the changes. You shouldn't rely on the data
being stored. Only non-critical data should be stored in the
meta-data, since apps that do not notify the database with these
functions will make you loose your data for the file. These functions will
return 0 if there was an error, or an error-code, for more descriptions of
errors, see \fBlibgnome/gnome-metadata.h\fR.
.P
Now if you actually want to use the meta-data to store information
about files, you will most likely use the functions \fBgnome_metadata_set\fR,
\fBgnome_metadata_remove\fR and \fBgnome_metadata_get\fR. Again these
functions return an integer, which is 0 in case there was no error,
or they use the same error codes as the previous functions.
.P
The functions work with a a key string for which they store a piece of
data. The data is represented by a size integer and a character pointer.
\fBgnome_metadata_set\fR takes the filename as the first argument, the name
or key of the data as the second argument, the size as the third and the
pointer to the actual data as the forth argument. This function just sets
that data for that file and key. \fBgnome_metadata_remove\fR will clear
a particular data item on a file, so it takes a file and then the key
name as the second argument. \fBgnome_metadata_get\fR takes the filename
as the first argument and the key name as the second, then it returns
data size though an integer pointer you pass though the third argument
and the actual data though a pointer to a pointer you pass as the fourth
argument. The data returned is newly allocated and should be freed after
use. Small example follows (in real life you should also check the return
of the functions for errors):
.VERBON 16
int size;
char *data;
\&...
/*set some bogus data on a file*/
gnome_metadata_set("/some/file/name","bogus",5,"BLAH");
\&...
/*retrieve the data back*/
gnome_metadata_get("/some/file/name","bogus",&size,&data);
.VERBOFF

.H 2 "Using the libgnomeui Library"
.P
The libgnomeui library is the library you'll be using most, it includes
the basic UI framework of your application and other X and GTK+ specific
things, such as session management, and many utility widgets. It also
contains the GnomeCanvas widget, which deserves separate treatment.
This is the library that makes the programmers life easy. With
plain GTK+ you have to do a lot of things by yourself, reinventing
the wheel every time, but this library takes care of the UI setup for
you and still allows the user to configure that behavior and have it
be consistent over different applications.

.H 3 "GnomeApp Widget Basics"
.H 4 "Overview"
.P
\fBGnomeApp\fR is the basic widget behind each app. It is the main window of
the application, containing the document being worked on and the applications
menus, tool-bars and status bars. It also remembers the docked positions
of menu bars and tool-bars and such for you so that the user gets the window
the way he left it when he left the application last time.

.H 4 "Creating"
.P
Creating a new \fBGnomeApp\fR widget is as easy as calling
\fBgnome_app_new\fR with the application name, which is usually the name
of the executable or something else that is unique to your application and
the title of the main window. Then you create the content of the main window
and add it to the \fBGnomeApp\fR widget by calling \fBgnome_app_set_contents\fR
with your contents as the argument.
.P
Adding menu-bars, tool-bars and status-bars is equally easy, you call
\fBgnome_app_set_toolbar\fR, \fBgnome_app_set_menus\fR or
\fBgnome_app_set_statusbar\fR. \fBgnome_app_set_toolbar\fR is for simple
applications that have only one tool-bar, for more complicated applications
you need to use \fBgnome_app_add_toolbar\fR, which allows you to add as
many docked tool-bars as you need.

.H 4 "Menu and Tool-bar Creation"
.P
Most of the time however, you don't really want to create your menu-bars
and tool-bars by yourself. You can use functions from
\fBlibgnomeui/gnome-app-helper.h\fR to construct menus and tool-bars for you.
All you need is to fill in a couple of structures with the your
information, and call \fBgnome_app_create_menus\fR or
\fBgnome_app_create_toolbar\fR with that structure and voila,
your application has menus and tool-bars. Sometimes you wish to pass
a data pointer to all the callbacks from those structures to work with,
then you'd use the \fBgnome_app_create_toolbar_with_data\fR and
\fBgnome_app_create_menus_with_data\fR, and pass an extra parameter which
will be passed in the data field of the callbacks. Here is the definition of
the structure you need to fill (actually you fill in an array of
such structures). Also note I included the enums that you will need
to fill that structure.
.VERBON 16
know what it does, you don't need it
/* These values identify the type of pixmap used in an item */
typedef enum {
	GNOME_APP_PIXMAP_NONE,		/* No pixmap specified */
	GNOME_APP_PIXMAP_STOCK,		/* Use a stock pixmap
					   (GnomeStock) */
	GNOME_APP_PIXMAP_DATA,		/* Use a pixmap from inline
					   xpm data */
	GNOME_APP_PIXMAP_FILENAME	/* Use a pixmap from the
					   specified filename */
} GnomeUIPixmapType;

/* This is the structure that defines an item in a menu bar
 * or tool-bar.  The idea is to create an array of such
 * structures with the information needed to create menus or
 * tool-bars.  The most convenient way to create such a structure
 * is to use the GNOMEUIINFO_* macros provided below. */
typedef struct {
	GnomeUIInfoType type;		/* Type of item */
	gchar *label;			/* String to use in the label */
	gchar *hint;			/* For tool-bar items, the
					   tool-tip. For menu items, the
					   status bar message */
	gpointer moreinfo;		/* For an item, toggle-item, or 
					   radio-item, this is a pointer
					   to the function to call when
					   the item is activated. For
					   a subtree, a pointer to
					   another array of GnomeUIInfo 
					   structures. For a radio-item
					   lead entry, a pointer to an
					   array of GnomeUIInfo
					   structures for the radio 
					   item group. For a help item, 
					   specifies the help node to
					   load (i.e. the application's
					   identifier) or NULL for the
					   main program's name.  For
					   builder data, points to the 
					   GnomeUIBuilderData structure
					   for the following items */
	gpointer user_data;		/* Data pointer to pass to
					   callbacks */
	gpointer unused_data;		/* Reserved for future expansion, 
					   should be NULL */
	GnomeUIPixmapType pixmap_type;	/* Type of pixmap for the item */
	gpointer pixmap_info;		/* Pointer to the pixmap
					 * information:
					 *
					 * For GNOME_APP_PIXMAP_STOCK, a 
					 * pointer to the stock icon name.
					 *
					 * For GNOME_APP_PIXMAP_DATA, a 
					 * pointer to the inline xpm data.
					 *
					 * For GNOME_APP_PIXMAP_FILENAME, a 
					 * pointer to the filename string.
					 */
	guint accelerator_key;		/* Accelerator key, or 0 for none */
	GdkModifierType ac_mods;	/* Mask of modifier keys for the 
					   accelerator */

	GtkWidget *widget;		/* Filled in by gnome_app_create*,
					   you can use this to tweak the
					   widgets once they have been
					   created */
} GnomeUIInfo;
.VERBOFF
Don't worry if you don't know all the items or what they mean. If you don't
know what it is, just leave it NULL or 0. Most of the time, it's easiest
to copy the menu's from another app and just modify them for your
needs, that way you will also know much better what does what then
by just looking at the structure.
.P
Sometimes some entries would be very simple, so one can just use one
of the simple macros provided. For example, for the end of a menu,
one would use the \fBGNOMEUIINFO_END\fR macro, for a separator one uses the
\fBGNOMEUIINFO_SEPARATOR\fR macro. Now for the actual items there
are also macros, which require you to fill in less info. For example
if you have an item that you provide an xpm format data for, you can 
use the \fBGNOMEUIINFO_ITEM(label, tooltip, callback, xpm_data)\fR
macro, where label is the text of the label, tool-tip is the tool-tip
that the user gets when he goes over that item (or it can be \fBNULL\fR),
callback is the function that gets called when the user presses that item,
and the xpm_data is a pointer to an xpm data you want to use as the
icon. If you have no icon you can just use the
\fBGNOMEUIINFO_ITEM_NONE(label, tooltip, callback)\fR macro. If what you
are adding is a standard item for which there is a stock icon (we'll talk
about those next), you can use the
\fBGNOMEUIINFO_ITEM_STOCK(label, tooltip, callback, stock_id)\fR macro
where the stock_id is the id of the stock icon you want to use.
Then for your
main menu bar, or to put sub-menus inside your menus, you can use
\fBGNOMEUIINFO_SUBTREE(label, tree)\fR and
\fBGNOMEUIINFO_SUBTREE_STOCK(label, tree, stock_id)\fR, where the
tree is the array of \fBGnomeUIInfo\fR structures that you want to use as
that sub-menu. There are a few other macros, but most of the time you will
get by with just these macros, so you don't need to learn the entire
structure of the \fBGnomeUIInfo\fR.
Here is a very simple application that makes use of these:
.VERBON 16
/*
 * A simple Gnome program, outside of GNOME tree, not using i18n
 * uiinfo.c
 */
/* the very basic gnome include */
#include <gnome.h>

/* a callback for the buttons */
static void
a_callback(GtkWidget *button, gpointer data)
{
	/*just print a string so that we know we got there*/
	g_print("Inside Callback\\n");
}

GnomeUIInfo file_menu[] = {
	GNOMEUIINFO_ITEM_NONE("_Menuitem","Just a menuitem",
			      a_callback),
	GNOMEUIINFO_ITEM_NONE("M_enuitem2","Just a menuitem",
			      a_callback),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK("E_xit","Exit the application",
			       gtk_main_quit,
			       GNOME_STOCK_MENU_EXIT),
	GNOMEUIINFO_END
};

GnomeUIInfo menubar[] = {
	GNOMEUIINFO_SUBTREE("_File",file_menu),
	GNOMEUIINFO_END
};

GnomeUIInfo toolbar[] = {
	GNOMEUIINFO_ITEM_STOCK("Exit","Exit the application",
			       gtk_main_quit,
			       GNOME_STOCK_PIXMAP_EXIT),
	GNOMEUIINFO_END
};

int
main(int argc, char *argv[])
{
	GtkWidget *app;
	GtkWidget *button;
	GtkWidget *hbox;
	GtkWidget *label;

	/* Initialize GNOME, this is very similar to gtk_init */
	gnome_init ("menu-basic-example", "0.1", argc, argv);
	
	/* Create a Gnome app widget, which sets up a basic
	   window for your application */
	app = gnome_app_new ("menu-basic-example",
			     "Basic GNOME Application");

	/* bind "delete_event", which is the event we get when
	   the user closes the window with the window manager,
	   to gtk_main_quit, which is a function that causes
	   the gtk_main loop to exit, and consequently to quit
	   the application */
	gtk_signal_connect (GTK_OBJECT (app), "delete_event",
			    GTK_SIGNAL_FUNC (gtk_main_quit),
			    NULL);

	/*make a label as the contents*/
	label = gtk_label_new("BLAH BLAH BLAH BLAH BLAH");

	/*add the label as contents of the window*/
	gnome_app_set_contents (GNOME_APP (app), label);

	/*create the menus for the application*/
	gnome_app_create_menus (GNOME_APP (app), menubar);

	/*create the tool-bar for the application*/
	gnome_app_create_toolbar (GNOME_APP (app), toolbar);

	/* show everything inside this app widget and the app
	   widget itself */
	gtk_widget_show_all(app);
	
	/* enter the main loop */
	gtk_main ();
	
	return 0;
}
.VERBOFF
Voila, an application with a menu and a tool-bar. As you see, adding
extra menu items is just adding extra definitions to the GnomeUIInfo
structure array.

.H 4 "Accelerator keys"
.P
You have probably noticed the underlines in the labels for the menu items,
these specify the accelerators for that menu. That's really all you need
to do to add accelerators for menu items. The way accelerators work
is very similar to the other windowing systems out there, \fBalt-<key>\fR
if you are not browsing the menus or just the \fB<key>\fR if you have the
menu open.

.H 3 "Stock Icons"
.P
Since most of the time you will want to use standard buttons and menu items
(such as \fBOpen\fR or \fBSave as...\fR), and you want to provide icons
with the menu items or tool-bar buttons or just dialog buttons, to make
it easier to navigate, you can use some of the predefined icons from
\fBgnome-libs\fR. These are called \fBStock Icons\fR. You have already
seen an example of how to use stock menu icons and regular stock icons
in menus and tool-bars (you just use the proper define from
\fBlibgnomeui/gnome-stock.h\fR). There are also stock buttons, where
you can get back a button widget based on a stock description.
.P
Here is a list of the \fInormal\fR gnome stock icons, these are regular
sized for use in tool-bars and other places where you need a normal sized
icon. They are given as defines of string constants and their meaning
should be obvious.
.VERBON 16
#define GNOME_STOCK_PIXMAP_NEW         "New"
#define GNOME_STOCK_PIXMAP_OPEN        "Open"
#define GNOME_STOCK_PIXMAP_CLOSE       "Close"
#define GNOME_STOCK_PIXMAP_REVERT      "Revert"
#define GNOME_STOCK_PIXMAP_SAVE        "Save"
#define GNOME_STOCK_PIXMAP_SAVE_AS     "Save As"
#define GNOME_STOCK_PIXMAP_CUT         "Cut"
#define GNOME_STOCK_PIXMAP_COPY        "Copy"
#define GNOME_STOCK_PIXMAP_PASTE       "Paste"
#define GNOME_STOCK_PIXMAP_PROPERTIES  "Properties"
#define GNOME_STOCK_PIXMAP_PREFERENCES "Preferences"
#define GNOME_STOCK_PIXMAP_HELP        "Help"
#define GNOME_STOCK_PIXMAP_SCORES      "Scores"
#define GNOME_STOCK_PIXMAP_PRINT       "Print"
#define GNOME_STOCK_PIXMAP_SEARCH      "Search"
#define GNOME_STOCK_PIXMAP_SRCHRPL     "Search/Replace"
#define GNOME_STOCK_PIXMAP_BACK        "Back"
#define GNOME_STOCK_PIXMAP_FORWARD     "Forward"
#define GNOME_STOCK_PIXMAP_FIRST       "First"
#define GNOME_STOCK_PIXMAP_LAST        "Last"
#define GNOME_STOCK_PIXMAP_HOME        "Home"
#define GNOME_STOCK_PIXMAP_STOP        "Stop"
#define GNOME_STOCK_PIXMAP_REFRESH     "Refresh"
#define GNOME_STOCK_PIXMAP_UNDO        "Undo"
#define GNOME_STOCK_PIXMAP_REDO        "Redo"
#define GNOME_STOCK_PIXMAP_TIMER       "Timer"
#define GNOME_STOCK_PIXMAP_TIMER_STOP  "Timer Stopped"
#define GNOME_STOCK_PIXMAP_MAIL	       "Mail"
#define GNOME_STOCK_PIXMAP_MAIL_RCV    "Receive Mail"
#define GNOME_STOCK_PIXMAP_MAIL_SND    "Send Mail"
#define GNOME_STOCK_PIXMAP_MAIL_RPL    "Reply to Mail"
#define GNOME_STOCK_PIXMAP_MAIL_FWD    "Forward Mail"
#define GNOME_STOCK_PIXMAP_MAIL_NEW    "New Mail"
#define GNOME_STOCK_PIXMAP_TRASH       "Trash"
#define GNOME_STOCK_PIXMAP_TRASH_FULL  "Trash Full"
#define GNOME_STOCK_PIXMAP_UNDELETE    "Undelete"
#define GNOME_STOCK_PIXMAP_SPELLCHECK  "Spellchecker"
#define GNOME_STOCK_PIXMAP_MIC         "Microphone"
#define GNOME_STOCK_PIXMAP_LINE_IN     "Line In"
#define GNOME_STOCK_PIXMAP_CDROM       "Cdrom"
#define GNOME_STOCK_PIXMAP_VOLUME      "Volume"
#define GNOME_STOCK_PIXMAP_BOOK_RED    "Book Red"
#define GNOME_STOCK_PIXMAP_BOOK_GREEN  "Book Green"
#define GNOME_STOCK_PIXMAP_BOOK_BLUE   "Book Blue"
#define GNOME_STOCK_PIXMAP_BOOK_YELLOW "Book Yellow"
#define GNOME_STOCK_PIXMAP_BOOK_OPEN   "Book Open"
#define GNOME_STOCK_PIXMAP_ABOUT       "About"
#define GNOME_STOCK_PIXMAP_QUIT        "Quit"
#define GNOME_STOCK_PIXMAP_MULTIPLE    "Multiple"
#define GNOME_STOCK_PIXMAP_NOT         "Not"
#define GNOME_STOCK_PIXMAP_CONVERT     "Convert"
#define GNOME_STOCK_PIXMAP_JUMP_TO     "Jump To"
#define GNOME_STOCK_PIXMAP_UP          "Up"
#define GNOME_STOCK_PIXMAP_DOWN        "Down"
#define GNOME_STOCK_PIXMAP_TOP         "Top"
#define GNOME_STOCK_PIXMAP_BOTTOM      "Bottom"
#define GNOME_STOCK_PIXMAP_ATTACH      "Attach"
#define GNOME_STOCK_PIXMAP_INDEX       "Index"
#define GNOME_STOCK_PIXMAP_FONT        "Font"
#define GNOME_STOCK_PIXMAP_EXEC        "Exec"

#define GNOME_STOCK_PIXMAP_ALIGN_LEFT  "Left"
#define GNOME_STOCK_PIXMAP_ALIGN_RIGHT "Right"
#define GNOME_STOCK_PIXMAP_ALIGN_CENTER "Center"
#define GNOME_STOCK_PIXMAP_ALIGN_JUSTIFY "Justify"

#define GNOME_STOCK_PIXMAP_TEXT_BOLD    "Bold"
#define GNOME_STOCK_PIXMAP_TEXT_ITALIC  "Italic"
#define GNOME_STOCK_PIXMAP_TEXT_UNDERLINE "Underline"
#define GNOME_STOCK_PIXMAP_TEXT_STRIKEOUT "Strikeout"

#define GNOME_STOCK_PIXMAP_EXIT        GNOME_STOCK_PIXMAP_QUIT
.VERBOFF
If you need to use these outside of \fBGnomeUIInfo\fR, you need
to get the widget with the pixmap. What you do is you call the
\fBgnome_stock_pixmap_widget\fR function with your main window
as the first argument (so that it can copy it's style) and
the icon name (one of the above defines) as the second argument.
It returns a new widget which you can just use as a pixmap.
.P
For menus you want to use the \fB_MENU_\fR variety of the stock
pixmaps. These are smaller and these should be the ones you use
for the stock menu items in your \fBGnomeUIInfo\fR definitions.
.VERBON 16
#define GNOME_STOCK_MENU_BLANK        "Menu_"
#define GNOME_STOCK_MENU_NEW          "Menu_New"
#define GNOME_STOCK_MENU_SAVE         "Menu_Save"
#define GNOME_STOCK_MENU_SAVE_AS      "Menu_Save As"
#define GNOME_STOCK_MENU_REVERT       "Menu_Revert"
#define GNOME_STOCK_MENU_OPEN         "Menu_Open"
#define GNOME_STOCK_MENU_CLOSE        "Menu_Close"
#define GNOME_STOCK_MENU_QUIT         "Menu_Quit"
#define GNOME_STOCK_MENU_CUT          "Menu_Cut"
#define GNOME_STOCK_MENU_COPY         "Menu_Copy"
#define GNOME_STOCK_MENU_PASTE        "Menu_Paste"
#define GNOME_STOCK_MENU_PROP         "Menu_Properties"
#define GNOME_STOCK_MENU_PREF         "Menu_Preferences"
#define GNOME_STOCK_MENU_ABOUT        "Menu_About"
#define GNOME_STOCK_MENU_SCORES       "Menu_Scores"
#define GNOME_STOCK_MENU_UNDO         "Menu_Undo"
#define GNOME_STOCK_MENU_REDO         "Menu_Redo"
#define GNOME_STOCK_MENU_PRINT        "Menu_Print"
#define GNOME_STOCK_MENU_SEARCH       "Menu_Search"
#define GNOME_STOCK_MENU_SRCHRPL      "Menu_Search/Replace"
#define GNOME_STOCK_MENU_BACK         "Menu_Back"
#define GNOME_STOCK_MENU_FORWARD      "Menu_Forward"
#define GNOME_STOCK_MENU_FIRST        "Menu_First"
#define GNOME_STOCK_MENU_LAST         "Menu_Last"
#define GNOME_STOCK_MENU_HOME         "Menu_Home"
#define GNOME_STOCK_MENU_STOP         "Menu_Stop"
#define GNOME_STOCK_MENU_REFRESH      "Menu_Refresh"
#define GNOME_STOCK_MENU_MAIL         "Menu_Mail"
#define GNOME_STOCK_MENU_MAIL_RCV     "Menu_Receive Mail"
#define GNOME_STOCK_MENU_MAIL_SND     "Menu_Send Mail"
#define GNOME_STOCK_MENU_MAIL_RPL     "Menu_Reply to Mail"
#define GNOME_STOCK_MENU_MAIL_FWD     "Menu_Forward Mail"
#define GNOME_STOCK_MENU_MAIL_NEW     "Menu_New Mail"
#define GNOME_STOCK_MENU_TRASH        "Menu_Trash"
#define GNOME_STOCK_MENU_TRASH_FULL   "Menu_Trash Full"
#define GNOME_STOCK_MENU_UNDELETE     "Menu_Undelete"
#define GNOME_STOCK_MENU_TIMER        "Menu_Timer"
#define GNOME_STOCK_MENU_TIMER_STOP   "Menu_Timer Stopped"
#define GNOME_STOCK_MENU_SPELLCHECK   "Menu_Spellchecker"
#define GNOME_STOCK_MENU_MIC          "Menu_Microphone"
#define GNOME_STOCK_MENU_LINE_IN      "Menu_Line In"
#define GNOME_STOCK_MENU_CDROM	      "Menu_Cdrom"
#define GNOME_STOCK_MENU_VOLUME       "Menu_Volume"
#define GNOME_STOCK_MENU_BOOK_RED     "Menu_Book Red"
#define GNOME_STOCK_MENU_BOOK_GREEN   "Menu_Book Green"
#define GNOME_STOCK_MENU_BOOK_BLUE    "Menu_Book Blue"
#define GNOME_STOCK_MENU_BOOK_YELLOW  "Menu_Book Yellow"
#define GNOME_STOCK_MENU_BOOK_OPEN    "Menu_Book Open"
#define GNOME_STOCK_MENU_CONVERT      "Menu_Convert"
#define GNOME_STOCK_MENU_JUMP_TO      "Menu_Jump To"
#define GNOME_STOCK_MENU_UP           "Menu_Up"
#define GNOME_STOCK_MENU_DOWN         "Menu_Down"
#define GNOME_STOCK_MENU_TOP          "Menu_Top"
#define GNOME_STOCK_MENU_BOTTOM       "Menu_Bottom"
#define GNOME_STOCK_MENU_ATTACH       "Menu_Attach"
#define GNOME_STOCK_MENU_INDEX        "Menu_Index"
#define GNOME_STOCK_MENU_FONT         "Menu_Font"
#define GNOME_STOCK_MENU_EXEC         "Menu_Exec"

#define GNOME_STOCK_MENU_ALIGN_LEFT     "Menu_Left"
#define GNOME_STOCK_MENU_ALIGN_RIGHT    "MEnu_Right"
#define GNOME_STOCK_MENU_ALIGN_CENTER   "Menu_Center"
#define GNOME_STOCK_MENU_ALIGN_JUSTIFY  "Menu_Justify"

#define GNOME_STOCK_MENU_TEXT_BOLD      "Menu_Bold"
#define GNOME_STOCK_MENU_TEXT_ITALIC    "Menu_Italic"
#define GNOME_STOCK_MENU_TEXT_UNDERLINE "Menu_Underline"
#define GNOME_STOCK_MENU_TEXT_STRIKEOUT "Menu_Strikeout"

#define GNOME_STOCK_MENU_EXIT     GNOME_STOCK_MENU_QUIT
.VERBOFF
If you are building the menu yourself and just want to get a menu-item
that's built with the stock icon and a label, you can use the
\fBgnome_stock_menu_item\fR convenience routine. It takes the stock
icon type (one of the defines above) as the first argument, and the menu
text as the second argument, and it returns a newly created menu-item
widget.
.P
Then there are stock buttons. These are for use in your dialogs (see the next
section).
.VERBON 16
#define GNOME_STOCK_BUTTON_OK     "Button_Ok"
#define GNOME_STOCK_BUTTON_CANCEL "Button_Cancel"
#define GNOME_STOCK_BUTTON_YES    "Button_Yes"
#define GNOME_STOCK_BUTTON_NO     "Button_No"
#define GNOME_STOCK_BUTTON_CLOSE  "Button_Close"
#define GNOME_STOCK_BUTTON_APPLY  "Button_Apply"
#define GNOME_STOCK_BUTTON_HELP   "Button_Help"
#define GNOME_STOCK_BUTTON_NEXT   "Button_Next"
#define GNOME_STOCK_BUTTON_PREV   "Button_Prev"
#define GNOME_STOCK_BUTTON_UP     "Button_Up"
#define GNOME_STOCK_BUTTON_DOWN   "Button_Down"
#define GNOME_STOCK_BUTTON_FONT   "Button_Font"
.VERBOFF
To get a button widget with the stock icon and text, you can just use the
function \fBgnome_stock_button\fR with the button type (one of the above
defines) as the argument. Now sometimes you want to create a mixture of
stock or ordinary buttons, what you can do is call the 
\fBgnome_stock_or_ordinary_button\fR function with either the type of
a stock button or just a text for the button label. The function checks
if it is one of the above strings, and if it's not it creates an
ordinary button widget with the text as the label.

.H 3 "Dialogs"
.H 4 "Generic Dialogs"
.P
If you need to create you own custom dialog, \fBgnome-dialog\fR is the 
way to do it. It can handle both modal and non-modal dialogs, although,
it's definitely much more friendly to the users of your program if
you use a non-modal dialog box, if at all possible, although non-modal
dialog boxes tend to have problems associated with them, and sometimes 
can cause strange bugs, for example if a non-modal dialog box is associated
with a window, you'd better bind the \fBdestroy\fR signal of the window
and set it to destroy the dialog box as well, since otherwise it could
hang around even though the window or document it was supposed to act on
is already dead. However modal dialogs (while definitely easier to program)
are usually pretty annoying to use, so avoid them if you at all can.
.P
To make a new \fBGnomeDialog\fR widget, just use the \fBgnome_dialog_new\fR
function. You pass the title of the dialog as the first argument, and then
multiple arguments as the button titles terminated by a NULL. The button
titles can also be the \fBGNOME_STOCK_BUTTON_*\fR definitions if you want
stock buttons on your dialog. Then you need to add content to the dialog,
the dialog is created with a vertical box (\fBGtkVBox\fR) for you to use,
just by using \fBGNOME_DIALOG(dialog)->vbox\fR. Into that you add your
content. At this point you have to decide if you want to do a modal dialog
or a non-modal dialog.
.P
In case you want to do a modal dialog, all you need to do is to call
\fBgnome_dialog_run_and_close\fR function and it will run the dialog,
wait for a user to press a button or close the dialog, and then close
the dialog. This function will return the number of the button that was
pressed or -1 if the dialog was just closed. In case you don't want to
close the dialog when just any button is pressed, you use the
\fBgnome_dialog_run\fR function, and after you get a result, do what you
need to do for that particular button press. Then if you want to run
the dialog more, you just loop back to \fBgnome_dialog_run\fR, and if
you want to close, you run \fBgnome_dialog_close\fR. Here's an example
of the second scheme.
.VERBON 16
GtkWidget *dlg;
int i;
\&...
/*create a new dialog, DON'T forget the NULL on the end,
  it is very important!*/
dlg = gnome_dialog_new("A Dialog",
		       GNOME_STOCK_BUTTON_OK,
		       GNOME_STOCK_BUTTON_APPLY,
		       GNOME_STOCK_BUTTON_CLOSE,
		       NULL);
\&...
/*add some content to the dialog here*/
\&...
/*set up an infinite loop*/
for(;;) {
	i = gnome_dialog_run(GNOME_DIALOG(dlg));
	if(i == 0 || i == 2) {
		/*the user pressed OK or close, so we will get
		  out of the loop and close the dialog, or the
		  user pressed */
		gnome_dialog_close(GNOME_DIALOG(dlg));
		break;
	} else if(i < 0) {
		/*the user closed the dialog from the window
		  manager*/
		break;
	} else if(i == 1) {
		/*user pressed apply we don't want to close*/
		\&...
	}
}
.VERBOFF
By default the dialog is destroyed when closed, so you don't have to worry
about it's destruction. You can change this behavior if you wish though.
.P
If you are doing a non-modal dialog box, things get a little more complicated.
You create the dialog as above, but then you bind the \fBclicked\fR signal
of the \fBGnomeDialog\fR widget. That signal has as it's second argument
the button number that was pressed. After that you should use the
\fBgnome_dialog_set_close\fR function to tell \fBGnomeDialog\fR that
we want to close the dialog when the user first presses any button, if
you want that behavior, otherwise you'll have to do \fBgnome_dialog_close\fR
in the \fBclicked\fR signal handler for the buttons you want to close on.
After that is set up you just \fBgtk_widget_show\fR the dialog. An
example follows:
.VERBON 16
/*the clicked signal handler*/
static void
dialog_clicked(GnomeDialog *dlg, int button, gpointer data)
{
	switch(button) {
	case 1:
		/*user pressed apply*/
		\&...
		return;
	case 0:
		/*user pressed OK*/
		\&...
		/*fall though to close*/
	case 2:
		/*user pressed close*/
		gnome_dialog_close(dlg);
		break;
	}
}

/*somewhere else in the source file*/
\&...
GtkWidget *dlg;
\&...
/*create a new dialog, DON'T forget the NULL on the end, it
  is very important!*/
dlg = gnome_dialog_new("A Dialog",
		       GNOME_STOCK_BUTTON_OK,
		       GNOME_STOCK_BUTTON_APPLY,
		       GNOME_STOCK_BUTTON_CLOSE,
		       NULL);
\&...
/*add some content to the dialog here*/
\&...
/*bind the clicked handler*/
gtk_signal_connect(GTK_OBJECT(dlg),"clicked",
		   GTK_SIGNAL_FUNC(dialog_clicked),
		   NULL);
/*show the dialog, note that this is not a modal dialog,
  so the program doesn't block here, but continues*/
gtk_widget_show(dlg);
.VERBOFF
This implements the same dialog as the modal example above, only non modal.

.H 4 "Message Box"
.P
\fBGnomeMessageBox\fR is an object derived from \fBGnomeDialog\fR. As such
you use it in the exact same manner, the only difference here is that
it automatically sets up the insides of the dialog to be a single label
and an icon of the selected message box type. The message box types are
as follows:
.VERBON 16
#define GNOME_MESSAGE_BOX_INFO      "info"
#define GNOME_MESSAGE_BOX_WARNING   "warning"
#define GNOME_MESSAGE_BOX_ERROR     "error"
#define GNOME_MESSAGE_BOX_QUESTION  "question"
#define GNOME_MESSAGE_BOX_GENERIC   "generic"
.VERBOFF
To create a message box, you use the function \fBgnome_message_box_new\fR
with the first argument being the message text, the second argument being the
type of the message box (one of the defines above), and then any number
of buttons terminated by a NULL exactly as in the \fBGnomeDialog\fR's case.
After created it is again used exactly the same as \fBGnomeDialog\fR.

.H 4 "Property Dialogs"
.P
If you have some properties to set in your application, you should use
a \fBGnomePropertyBox\fR dialog for the preferences to make the
applications more consistent. Again this object is derived from
\fBGnomeDialog\fR so it's use is similar. But \fBGnomePropertyBox\fR
defines some new signals, namely \fBapply\fR and \fBhelp\fR. They both
get passed the page number as the second argument. For help you should
use this to display the proper help page, however for apply, this was
created for adding a per-page apply button, which was not realized yet,
so you should ignore any \fBapply\fR signal with the page number other
then -1, which is the \fIglobal\fR apply. This can be done with a simple
if statement at the top of your apply routine. You can choose to be
per-page apply \fIready\fR, by doing a per-page apply in your code, but
it is not sure if this code will ever get completed. It should be
safe to do just the global apply as that is the only thing implemented
in \fBgnome-libs 1.0\fR.
.P
To use property dialogs, you call \fBgnome_property_box_new\fR, which will
create a completely new dialog for you with a notebook and the four buttons.
\fBOK\fR, which will call your apply handler for all pages and then for the
-1 page, and then it will close the dialog, \fBApply\fR, which will call the
apply handler for all pages and then for the -1 page, \fBClose\fR, which will
just close the dialog, and \fBHelp\fR which will call your help handler if
you bound it. You then connect the \fBapply\fR signal to your apply handler,
and most likely the \fBdestroy\fR signal on the property box to destroy the
data associated with the property box when it closes. You then create the
different pages for your property box and add them with,
\fBgnome_property_box_append_page\fR, which takes your page as the second
argument and a label as the third (usually this will be just a
\fBGtkLabel\fR). You also want to connect the different signals for the
widgets on your pages, to mark the property box as changed (otherwise
the Apply and OK buttons will not be sensitive). You do this by calling
\fBgnome_property_box_changed\fR every time the user changed something with
the widgets. For example on entry (and derived) widgets you connect to the
\fBchanged\fR signal. Example follows:
.VERBON 16
/*apply handler*/
static void
property_apply(GnomePropertyBox *box, int page_num, gpointer data)
{
	/*ignore page numbers other then -1*/
	if(page_num!=-1)
		return;
	/*do your apply routine here*/
	\&...
}
\&...
/*somewhere else in the source file*/
GtkWidget *pbox;
GtkWidget *widget;
\&...
pbox = gnome_property_box_new();
gtk_signal_connect(GTK_OBJECT(pbox),"apply",
		   GTK_SIGNAL_FUNC(property_apply),NULL);
\&...
/*you create a page for the property box and added it to the
  container called widget*/
gnome_property_box_append_page(GNOME_PROPERTY_BOX(pbox),
			       widget,
			       gtk_label_new("SomePage"));
/*then add other pages in similar manner*/
\&...
/*we show the dialog box*/
gtk_widget_show_all(pbox);
.VERBOFF

.H 4 "File Picking Dialog"
.P
Gnome doesn't have it's own file picking dialog, although this is planned
for the future, for now you need to use the regular \fBGTK+\fR file
dialog.
.P
Use of the file dialog is very simple. You create the dialog with
\fBgtk_file_selection_new\fR, passing it the title of the dialog box
as the argument. After this you bind the clicked signal on the \fBOK\fR
and \fBCancel\fR buttons. For example for a loading dialog box, you could
check that the file is of the correct type when the user presses OK and
if so then close the dialog (usually with \fBgtk_widget_destroy\fR). Or
for saving dialog, you could ask if the file exists. File selection
dialog boxes are usually safe and simple to do non-modal. Just make sure
you'd destroy the file dialog box when the object or window it's supposed
to work with. Here's the routine that invokes the save as dialog for
\fBAchtung\fR, which is a presentation program we're working on.
.VERBON 16
void
presentation_save_as (AchtungPresentation *p)
{
	GtkFileSelection *fsel;
	
	g_return_if_fail (p != NULL);
	g_return_if_fail (p->doc != NULL);

	fsel = (GtkFileSelection *)
		gtk_file_selection_new (_("Save presentation as"));
	if (p->real_file && p->filename)
		gtk_file_selection_set_filename (fsel, p->filename);

	gtk_object_set_data(GTK_OBJECT(fsel),"p",p);
	
	/* Connect the signals for Ok and Cancel */
	gtk_signal_connect (GTK_OBJECT (fsel->ok_button), "clicked",
			    GTK_SIGNAL_FUNC (save_ok), fsel);
	gtk_signal_connect_object
		(GTK_OBJECT (fsel->cancel_button), "clicked",
		 GTK_SIGNAL_FUNC (gtk_widget_destroy), 
		 GTK_OBJECT(fsel));

	gtk_window_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);

	/*if the presentation dies so do it's dialogs*/
	gtk_signal_connect_object_while_alive
		(GTK_OBJECT (p), "destroy",
		 GTK_SIGNAL_FUNC (gtk_widget_destroy), 
		 GTK_OBJECT(fsel));

	gtk_widget_show (GTK_WIDGET (fsel));
}
.VERBOFF
This is actually a save_as method for \fBAchtungPresentation\fR object
in object oriented speak.
\fBAchtungPresentation\fR is a GtkObject we use for storing all the
presentation data (This is a nice example of how to use GtkObject for
things not directly related to widgets or GUI programming). First we
check the arguments to the function with \fBg_return_if_fail\fR which
is for debugging purposes. Then we create a new \fBGtkFileSelection\fR
with a title of "Save presentation as". Ignore the \fB_()\fR macro around
the string for now, it's used for internationalization.
Afterwards we check if the presentation already has
a filename associated with it, and if so we set the filename on the
file selection dialog to that. After that we connect the the \fBOK\fR
button to a routine called \fBsave_ok\fR defined elsewhere in the file
and pass the file selection dialog as a
data argument. Then we use \fBconnect_object\fR to bind the \fBCancel\fR button
to destroying the file selection dialog. The \fBconnect_object\fR method
is similar to regular \fBconnect\fR but when it calls the function itself
it will pass the object from the data field as the first argument of the
function. So connecting to \fBgtk_widget_destroy\fR will destroy the object
passed in the data field, which is the file selection dialog. Then we
position the dialog near the mouse button. In the future when this dialog
is derived from \fBGnomeDialog\fR, you will not need to and actually should 
not do that, as that will be done according to use preferences as for all
the other gnome dialogs. After this we use yet another signal connection
method ... this time \fBgtk_signal_connect_object_while_alive\fR, which is
similar to \fBconnect_object\fR, but has a nice twist to it. The signal
will be disconnected when the object passed in the data field dies. This
needs to happen as the file dialog will most likely be destroyed before the
the presentation itself is, then when the presentation is destroyed itself,
it would try to destroy an already non-existent file selection dialog and
most likely cause a segmentation fault and crash. This way it is safe and
if the file selection dialog is still around when the presentation is
destroyed, it is destroyed with it.

.H 3 "Entries"
.P
Sometimes, especially in properties dialogs, you want fields for entering
text, files, pixmaps, icons or double precision numbers. This is what the
\fBgnome-*entry\fR widgets do.

.H 4 "GnomeEntry"
.P
This is an entry for regular text, but it includes history of previously
entered values. Note that this widget is not derived from \fBGtkEntry\fR,
but owns such a widget. This means that you can't use \fBGtkEntry\fR
methods on this object directly, but you need to get a pointer to
the \fBGtkEntry\fR object inside \fBGnomeEntry\fR. When you call
\fBgnome_entry_new\fR, you pass a \fBhistory_id\fR string to it. This
is a unique identifier to identify this entry, or this type of
entries in your application. All the entries that share this \fBhistory_id\fR
will have common history of values. After you create a \fBGnomeEntry\fR
you use the \fBgnome_entry_gtk_entry\fR function to get a pointer to the
\fBGtkEntry\fR object inside and bind any signals or manipulate text with
that instead. Here is an example:
.VERBON 16
GtkWidget *gnome_e;
GtkWidget *gtk_e;
\&...
gnome_entry_new("text1");
gtk_e = gnome_entry_gtk_entry(GNOME_ENTRY(gnome_e));
gtk_signal_connect(GTK_OBJECT(gtk_e),"changed",
		   GTK_SIGNAL_FUNC(entry_changed), NULL);
.VERBOFF

.H 4 "GnomeFileEntry"
.P
\fBGnomeEntry\fR is a basis for \fBGnomeFileEntry\fR. Again it is not derived,
but \fBGnomeEntry\fR is owned by \fBGnomeFileEntry\fR. This type of hierarchy
is throughout all the gnome entry widgets. \fBGnomeFileEntry\fR adds a browse
button on the right side of the entry, and also accepts file drops from the
file manager for example. It's use is extremely similar to \fBGnomeEntry\fR.
You create the entry with \fBgnome_file_entry_new\fR. The first argument
is the \fBhistory_id\fR of the \fBGnomeEntry\fR, and the second argument
is the title of the browse dialog box. To get the \fBGtkEntry\fR, you
again use the gtk_entry method, named \fBgnome_file_entry_gtk_entry\fR.
To finally get the filename, you can get the exact text from the
\fBGtkEntry\fR, or you might use a convenience method,
\fBgnome_file_entry_get_full_path\fR, which takes a flag
\fBfile_must_exist\fR as it's second argument. If this flag is set,
the function returns NULL if the file doesn't exists. If the flag
is not set or the file does exist, the function returns the full
path to the file.

.H 4 "GnomePixmapEntry"
.P
This is an entry for entering pixmaps (Images) of any size. It again
includes (not derives from) \fBGnomeFileEntry\fR, so it can do everything
the file entry can (including accepting drops). However this entry adds
a preview box for the pixmap above the entry. Also it's file selection
dialog includes a preview box to the right side of the file list.
It's use is again very similar to the entries above. You call
\fBgnome_pixmap_entry_new\fB with the same arguments as \fBGnomeFileEntry\fR,
with an added flag, \fBdo_preview\fR. This flag specifies if the preview
box is visible or not. But be careful, it doesn't save memory not to show
the preview, it just saves space. Again you use a
\fBgnome_pixmap_entry_gtk_entry\fR to get the \fBGtkEntry\fR widget.
To get a filename of the the pixmap, if it could be loaded as an image
for the preview (using imlib), you can use
\fBgnome_pixmap_entry_get_filename\fR, which returns NULL if the pixmap
files doesn't exist or could not be loaded, and the full filename otherwise.

.H 4 "GnomeIconEntry"
.P
The icon entry is very similar to the \fBGnomePixmapEntry\fR, but it is meant
for images in the standard 48x48 icon size. Also instead of the
preview box, there is a button with the image scaled to 48x48. If you
press the button you get a listing of images from the same directory as
the current icon. To create an icon entry use \fBgnome_icon_entry_new\fR
with \fBhistory_id\fR and \fBbrowse_dialog_title\fR string arguments. Once
you need an existing icon that is a real image, you use
\fBgnome_icon_entry_get_filename\fR which works just like
\fBgnome_pixmap_entry_get_filename\fR. You can also get the \fBGtkEntry\fR
by using \fBgnome_icon_entry_gtk_entry\fR.

.H 4 "GnomeNumberEntry"
.P
\fBGnomeNumberEntry\fR is an entry widget for entering double precision
numbers with a calculator. Most of the time for number entries you want
to use the \fBGtkSpinButton\fR widget, however for applications such
as mortgage calculators, or finance programs, where calculations are
necessary, you will want to use this entry type. Basically it's
a \fBGnomeEntry\fR widget with a button on the right side of it
which calls up a dialog with a calculator. The user can use the calculator
and press OK and the number entry is updated to what it was on the
calculator. To create a number entry widget, just use
\fBgnome_number_entry_new\fR, passing it the \fBhistory_id\fR as the
first argument and the title of the calculator dialog as the second
argument. To get the \fBGtkEntry\fR widget just use 
\fBgnome_number_entry_gtk_entry\fR. To get the number as a
\fBdouble\fR value, use \fBgnome_number_entry_get_number\fR method.

.H 3 "Using Images"
.P
When you need to use images in your apps, most likely you'll want the
\fBGnomePixmap\fR widget. It's advantage is that it makes using images
much easier without having to learn imlib, which is the image library
used by this widget.
.P
There are numerous \fBnew\fR functions for \fBGnomePixmap\fR, depending
on the source of the pixmap. The most used will probably be
\fBgnome_pixmap_new_from_file\fR which takes a filename which is an
image loadable by imlib and creates a pixmap widget for you.
There is also \fBgnome_pixmap_new_from_file_at_size\fR to which you
pass also the size to which the image should be scaled. If you have
already loaded the image with imlib (in case you wanted to do other
things to the pixmap first), you can use \fBgnome_pixmap_new_from_imlib\fR
and \fBgnome_pixmap_new_from_imlib_at_size\fR. Which take a
\fBGdkImlibImage\fR as the first argument. If you already have a
pixmap widget and want to change the image inside it, you can use
the \fBgnome_pixmap_load_*\fR which have almost the same syntax
as the new functions, except that you pass the \fBGnomePixmap\fR
as the first argument, and then the rest of the arguments as above,
and of course replace the _new_from_ for _load_.
.P
Here's an example of it's use:
.VERBON 16
GtkWidget *pix;
\&...
/*load somefile.png and scale it to 48x48*/
pix = gnome_pixmap_new_from_file_at_size("somefile.png",48,48);
/*now you can pack pix somewhere just like any other widget*/
\&...
/*now we want to change the files to otherfile.png and do no
  scaling*/
gnome_pixmap_load_file(GNOME_PIXMAP(pix),"otherfile.png");
.VERBOFF

.H 3 "Session Management"
.P
Your app should be able to save it's settings and restore them when
the user restarts your application, it should also be able to do
this for several different sessions. For instance the user might have a
normal session, but sometimes log into a special session where he has
different settings in applications. \fBgnome-libs\fR actually hides
the ugly details of this. For the most part you do not need to worry about
the real details of session management, unless you wish to do something
very clever or if your app does some complicated state saving. To do
simple session saving all you need is the following code (mostly taken
from gnome-hello-4-SM example program):
.VERBON 16
/*the save_yourself handler, you can safely ignore most of the
  parameters, and just save your session and return TRUE*/
static int 
save_yourself(GnomeClient *client, int phase,
	      GnomeSaveStyle save_style, int shutdown,
	      GnomeInteractStyle interact_style, int fast,
	      gpointer client_data)
{
	/*get the prefix for our config*/
	char *prefix= gnome_client_get_config_prefix (client);

	/*this is a "discard" command for discarding data from
	  a saved session, usually this will work*/
	char *argv[]= { "rm", "-r", NULL };

	/* Save the state using gnome-config stuff. */
	gnome_config_push_prefix (prefix);

	gnome_config_set_int("Section/Key",some_value);
	\&...
	gnome_config_pop_prefix ();
	gnome_config_sync();

	/* Here is the real SM code. We set the argv to the
	   parameters needed to restart/discard the session that
	   we've just saved and call the
	   gnome_session_set_*_command to tell the session
	   manager it. */
	argv[2]= gnome_config_get_real_path (prefix);
	gnome_client_set_discard_command (client, 3, argv);

	/* Set commands to clone and restart this application.
	   Note that we use the same values for both -- the
	   session management code will automatically add
	   whatever magic option is required to set the session
	   id on startup. The client_data was set to the
	   command used to start this application when
	   save_yourself handler was connected. */
	argv[0]= (gchar*) client_data;
	gnome_client_set_clone_command (client, 1, argv);
	gnome_client_set_restart_command (client, 1, argv);
	
	return TRUE;
}

static void
die (GnomeClient *client, gpointer client_data)
{
	/* Just exit in a friendly way.  We don't need to
	   save any state here, because the session manager
	   should have sent us a save_yourself-message
	   before.  */
	gtk_exit (0);
}

\&...
GnomeClient *client;
\&...
/*this is somewhere in your main function presumably.
  make sure this is done AFTER the gnome_init call!*/

/* Get the master client, that was hopefully connected to the
   session manager int the 'gnome_init' call.  All communication
   to the session manager will be done with this master client. */
client = gnome_master_client ();

/* Arrange to be told when something interesting happens.  */
gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
		    GTK_SIGNAL_FUNC (save_yourself),
		    (gpointer) argv[0]);
gtk_signal_connect (GTK_OBJECT (client), "die",
		    GTK_SIGNAL_FUNC (die), NULL);

/*check if we are connected to a session manager*/
if (GNOME_CLIENT_CONNECTED (client)) {
	/*we are connected, we will get the prefix under which
	  we saved our session last time and load up our data*/
	gnome_config_push_prefix
		(gnome_client_get_config_prefix (client));

	some_value = gnome_config_get_int("Section/Key=0");

	gnome_config_pop_prefix ();
} else {
	/*we are not connected to any session manager, here you
	  will just initialize your session like you normally
	  do without a session manager*/
	\&...
}
.VERBOFF
This is a very simple session management which will be enough for most
programs, for more information on session management, you should consult
the gnome developer documentation which should be available by now.

.H 3 "Multiple Document Interface"
.H 4 "The Main MDI Window"
.P
If your app handles documents, most likely you will want it to handle
multiple documents at one time. Gnome provides an MDI model that
is customizable by the user and simple to use. They can use three models
of the document display. Either a notebook style which is the most
useful one, where documents can be docked in notebooks, and can be
dragged out into separate windows if desired. Or a toplevel style where
each document is a separate toplevel window. Or finally a modal
style where there is only one window and the documents must be switched
though a menu. (Note that the examples here are taken from the
\fBgnome-hello-7-mdi\fR example app in \fBgnome-libs\fR, slightly
modified)
.P
To use the MDI features. You basically replace the the \fBgnome_app_new\fR
call with \fBgnome_mdi_new\fR with the same arguments as \fBgnome_app_new\fR.
To add menus and tool-bar, you use \fBgnome_mdi_set_menubar_template\fR and
\fBgnome_mdi_set_toolbar_template\fR with the GnomeUIInfo as the argument.
For MDI, these aren't the actual menus, as it will add it's own items to
the menus of each child. After this you set where the menu additions take
place. You call \fBgnome_mdi_set_child_menu_path\fR to the toplevel menu
name after which the child's own menus are inserted. This is the "File"
menu in most cases. Then you want to specify the path (menu name) to the
menu into which you want to insert a list of the children, you do this
by calling \fBgnome_mdi_set_child_list_path\fR with the name of the menu
and add a '/' on the end of it to specify that you want to insert those
items into the menu, not after the menu. Example:
.VERBON 16
GtkWidget *mdi;
\&...
mdi = gnome_mdi_new("gnome-hello-7-mdi", "GNOME MDI Hello");
\&...
/*main_menu and toolbar_info are the menu and tool-bar
  descriptions*/
gnome_mdi_set_menubar_template(mdi, main_menu);
gnome_mdi_set_toolbar_template(mdi, toolbar_info);

/* and document menu and document list paths (see
   gnome-app-helper menu insertion routines for details)  */
gnome_mdi_set_child_menu_path(GNOME_MDI(mdi), "File");
gnome_mdi_set_child_list_path(GNOME_MDI(mdi), "Children/");
.VERBOFF
In our GnomeUIInfo structures we have defined a menu named "File" and a
menu named "Children". The children menu was not given any items, it's
just an empty menu.
.P
Then you should open the main toplevel window with
\fBgnome_mdi_open_toplevel\fR. This will open a toplevel window without
any children. If you wish to use MDI's session management functionality,
you can define a function that creates a child given it's name. This is done
with the \fBgnome_mdi_restore_state\fR method, which takes the config path
as the second argument and a function pointer to a function which takes
a string and returns a new \fBGnomeMDIChild\fR widget (a widget sub-classed
from \fBGnomeMDIChild\fR actually). Say for example you are using the
session management shown above, so you could use:
.VERBON 16
gnome_config_push_prefix (gnome_client_get_config_prefix (client));
restart_ok = gnome_mdi_restore_state(GNOME_MDI(mdi), "MDI Session",
				     my_child_new_from_config);
gnome_config_pop_prefix ();
.VERBOFF
The restart_ok is a boolean value telling you if the loading actually loaded
all the data correctly.
.P
You should also bind the \fBdestroy\fR signal of the mdi object to do
\fBgtk_main_quit\fR when the mdi is destroyed.

.H 4 "The MDI Children"
.P
For complicated apps, all children should be derived from the virtual
\fBGnomeMDIChild\fR object. For simple apps, you don't need to derive
a new object, you can just use the \fBGnomeMDIGenericChild\fR, and use the
fact that you can store arbitrary data on arbitrary \fBGtkObject\fRs to
store your own data on the object.
.P
To use the generic child object, you create it with
\fBgnome_mdi_generic_child_new\fR to which you pass the name of the
child. When you get the object, you will need to set it up for your
use. First you add a function for creating new views of the same data.
A view is just a different window displaying the same file or data.
This is done with a call to \fBgnome_mdi_generic_child_set_view_creator\fR
to which you pass a pointer to a creator function which takes the child
widget and a data pointer as arguments and returns a data widget, which
is not the actual child widget, but actually the child of the
\fBGnomeMDIGenericChild\fR widget. After this you set the template for
the child's menus with \fBgnome_mdi_child_set_menu_template\fR, to which
you pass the \fBGnomeUIInfo\fR array pointer of the child menu definitions.
Then you should call \fBgnome_mdi_generic_child_set_config_func\fR to set
a function which returns a newly allocated string to save in the config
file. This string will be used to load up the child next time you
start and do the \fBgnome_mdi_restore_state\fR call. It should probably
be a filename of the document, or some string from which you can completely
recreate that window/document. Then you need to call
\fBgnome_mdi_generic_child_set_label_func\fR with a pointer to
a function that takes the \fBGnomeMDIGenericChild\fR as the first argument,
the old label widget pointer as the second argument, which would be null
if no label widget was yet set, and a data argument. This function can either
create a new label and destroy the old one, or just set the label if the
label exists. The label can be any widget, for example the
\fBgnome-hello-7mdi\fR example code uses a horizontal box widget into which
it adds a pixmap and a gtk label. After this if you need to add the child
to the mdi yourself, if you are loading a new file for example, you use
\fBgnome_mdi_add_child\fR and \fBgnome_mdi_add_view\fR, to add a new child
and a new view to the mdi. If you are creating a new child from the
\fBgnome_mdi_restore_state\fR function, you should just return the child,
the mdi will take care of adding it and adding the appropriate views.
You also probably want to set some data on the child widget at this
time to store your data with the object.
.P
Here's a short example of creating a new child, for a more complete example
you should look at the \fBgnome-hello-7-mdi\fR included with this tutorial.
.VERBON 16
GnomeMDI *mdi;
\&...	
GnomeMDIGenericChild *child;
\&...	
/*create a new child named 'name'*/
if((child = gnome_mdi_generic_child_new(name)) != NULL) {
	/*creator of a view*/
	gnome_mdi_generic_child_set_view_creator
		(child, my_child_create_view, NULL);
	/*set a menu template for child menu*/
	gnome_mdi_child_set_menu_template
		(GNOME_MDI_CHILD(child), main_child_menu);
	/*set function to get config string*/
	gnome_mdi_generic_child_set_config_func
		(child, my_child_get_config_string, NULL);
	/*set function that sets or creates a label*/
	gnome_mdi_generic_child_set_label_func
		(child, my_child_set_label, NULL);

	/* add the child to MDI */
	gnome_mdi_add_child(mdi, GNOME_MDI_CHILD(child));

	/* and add a new view of the child */
	gnome_mdi_add_view(mdi, GNOME_MDI_CHILD(child));
}
.VERBOFF

.H 2 "GnomeCanvas Widget"
.P
While \fBGnomeCanvas\fR widget is inside the libgnomeui library, it definitely
deserves a separate chapter. The canvas is a very high level high performance
graphics drawing widget and on top of that it's easy to use. It includes
support for both Xlib drawn graphics, which is faster especially over the
network, and anti-aliased drawing for better looking results.

.H 3 "Creating a Canvas Widget"
.P
To create a gnome canvas widget, you call the \fBgnome_canvas_new\fR. You
need to make sure that the canvas is created with a proper visual and
colormap. For example if you wish to draw imlib images inside it, you should
do this:
.VERBON 16
GtkWidget *canvas;
\&...
gtk_widget_push_visual(gdk_imlib_get_visual());
gtk_widget_push_colormap(gdk_imlib_get_colormap());
canvas = gnome_canvas_new();
gtk_widget_pop_visual();
gtk_widget_pop_colormap();
.VERBOFF
After this you also want to call \fBgnome_canvas_set_pixels_per_unit\fR
to set the scale of the canvas. You can then do \fBgtk_widget_set_usize\fR
to set the size of the widget, and \fBgnome_canvas_set_scroll_region\fR
to set the region in which you can scroll around in, this is given in
(x1, y1, x2, y2). Basically it's the outer
limits of your drawing. So once the canvas was created, you could do:
.VERBON 16
GnomeCanvas *canvas;
\&...
/*already created a canvas, now set it up*/
gnome_canvas_set_pixels_per_unit(canvas,10);
gnome_canvas_set_scroll_region(canvas,0.0,0.0,50.0,50.0);
.VERBOFF

.H 3 "Groups and Items"
.P
In the canvas there are items, the actual objects that are on the canvas,
and groups, which are just groupings of items. A group is actually derived
from a base \fBGnomeCanvasItem\fR object, this is useful to applying
functions to all the items inside the group. Such as moving or hiding
the entire group. There is also one default group, the root group.
You can get this group by calling \fBgnome_canvas_root\fR.

.H 3 "Creating Items"
.P
Creating items is slightly different usual. It's using the standard
GTK+ object model argument mechanism. Basically you call
\fBgnome_canvas_item\fR, with the parent canvas group as the first argument,
the type of object as the second argument, and then arguments given in
pairs (argument, value), terminated with a NULL. This is best illustrated by
an example:
.VERBON 16
GnomeCanvas *canvas;
GnomeCanvasItem *item;
\&...
item = gnome_canvas_item_new(gnome_canvas_root(canvas),
			     GNOME_TYPE_CANVAS_RECT,
			     "x1", 1.0,
			     "y1", 1.0,
			     "x2", 23.0,
			     "y2", 20.0,
			     "fill_color", "black",
			     NULL);
.VERBOFF
Note that it's extremely important that the value be the exact type, since
the compiler won't do the cast for you. If you're doing any calculations and
aren't sure that you get the right type, just cast it. I believe most if not
all numbers for canvas items are doubles.
.P
To find out the arguments that each item takes, consult the gnome
documentation or look into the \fBlibgnomeui/gnome-canvas*.h\fR header files.
They contain a table at the top of the file just like the one
that follows (which was taken from
\fBlibgnomeui/gnome-canvas-rect-ellipse.h\fR).
.DS
.fi
.P
For example here are arguments for rectangle (GNOME_TYPE_CANVAS_RECT) and
ellipse (GNOME_TYPE_CANVAS_ELLIPSE):

.TS H
center;
lb | lb | lb | lbw(2.5i).
Name	Type	Read/Write	Description
_
.TH
.T&
l | l | l | l.
x1	double	RW	T{
.fi
Leftmost coordinate of rectangle or ellipse
T}
y1	double	RW	T{
.fi
Topmost coordinate of rectangle or ellipse
T}
x2	double	RW	T{
.fi
Rightmost coordinate of rectangle or ellipse
T}
y2	double	RW	T{
.fi
Bottommost coordinate of rectangle or ellipse
T}
fill_color	string	W	T{
.fi
X color specification for fill color, or NULL pointer for
no color (transparent)
T}
fill_color_gdk	GdkColor*	RW	T{
.fi
Allocated GdkColor for fill
T}
outline_color	string	W	T{
.fi
X color specification for outline color, or NULL pointer for
no color (transparent)
T}
outline_color_gdk	GdkColor*	RW	T{
.fi
Allocated GdkColor for outline
T}
fill_stipple	GdkBitmap*	RW	T{
.fi
Stipple pattern for fill
T}
outline_stipple	GdkBitmap*	RW	T{
.fi
Stipple pattern for outline
T}	
width_pixels	uint	RW	T{
.fi
Width of the outline in pixels.  The outline will not be scaled
when the canvas zoom factor is changed.
T}	
width_units	double	RW	T{
.fi
Width of the outline in canvas units.  The outline will be scaled
when the canvas zoom factor is changed.
T}	
.TE
.TB "Arguments for rectangle and ellipse canvas items"
.DE

.P
Now suppose we want to change some of these properties. This is done with
a call to \fBgnome_canvas_item_set\fR. The first argument to this
function is the canvas item object pointer. The next arguments are the
same argument pairs as above when creating a new canvas object. For example
if we want to set the color to red on the rectangle we created above,
we can do this:
.VERBON 16
GnomeCanvas *canvas;
GnomeCanvasItem *item;
\&...
gnome_canvas_item_set(item
		      "fill_color", "red",
		      NULL);
.VERBOFF
.P
Then there are item methods for other operations on items. For example
the \fBgnome_canvas_item_move\fR method will take the x and y as 
second and third argument, and will move the item relative to it's
current position by x and y. Or the \fBgnome_canvas_item_hide\fR
and \fBgnome_canvas_item_show\fR, which hide and show the item,
respectively. To control the z order of the items, you can use
the methods \fBgnome_canvas_item_raise_to_top\fR and
\fBgnome_canvas_item_lower_to_bottom\fR to raise or lower the
item to the top or bottom of it's parent group's z order. To have finer
control over z order you can use the \fBgnome_canvas_item_raise\fR and
\fBgnome_canvas_item_lower\fR methods which take an extra integer 
argument which is 1 or larger, and specifies the number of levels
the item should move in the z order.

.H 3 "Anti-aliasing canvas"
.P
To create a canvas which uses anti aliasing for rendering of it's items,
instead of \fBgnome_canvas_new\fR function, you should use the
\fBgnome_canvas_new_aa\fR. You should also use the \fBGdkRgb\fR
visual and colormap. So you would do this to create a new anti-aliased
canvas:
.VERBON 16
GtkWidget *canvas;
\&...
gtk_widget_push_visual (gdk_rgb_get_visual ());
gtk_widget_push_colormap (gdk_rgb_get_cmap ());
canvas = gnome_canvas_new_aa ();
gtk_widget_pop_colormap ();
gtk_widget_pop_visual ();
.VERBOFF
After this you can use the canvas in exactly the same manner as the normal
canvas.
.P
Anti-aliased canvas items can generally do more then normal canvas items.
This is because of limitations of Xlib as a graphics library. It can for
example do any kind of affine transformation on it's objects, where on
and Xlib canvas you can only do affine transformations on some objects.

.H 2 "Drag and Drop"
.P
While drag and drop belongs into GTK+ itself, I thought it would be better
to cover it after some parts of GNOME were discussed.

.H 3 "Accepting Drops"
.P
You have already seen one drop handler back when we were discussing the
mime types. Basically, to accept drops, you have to decide which
mime type of data you want to be able to receive. You have already seen
one for "text/uri-list". Basically your handler will only receive data
of those mime types that you specify, so you only need to know how to
decode those.
.P
To specify the mime types you want to receive, you create an array of
\fBGtkTargetEntry\fR structures, where the first element is a string
of mime type, the second is an integer flag and the third is an integer
info. You can leave the flags at 0. The info field can be used if you
have several entries you are accepting, as the info integer will be passed
to your drop handler, so you can create a switch statement to handle
the different types of data. If you have only one type, just leave this
at 0.
.P
After this you need to set up the widget for dragging. You do
this by calling the \fBgtk_drag_dest_set\fR function. The first argument
is the widget you want to set up, the second is a flags argument for setting
up which types of default drag behavior to use, you can leave this at
\fBGTK_DEST_DEFAULT_ALL\fR. The next argument is the array of
\fBGtkTargetEntry\fR structures, the next argument is the number of items
in that array. The last argument is the type of action that you accept.
The types can be any of the following ORed together:
\fBGDK_ACTION_DEFAULT\fR, \fBGDK_ACTION_COPY\fR, \fBGDK_ACTION_MOVE\fR,
\fBGDK_ACTION_LINK\fR, \fBGDK_ACTION_PRIVATE\fR and \fBGDK_ACTION_ASK\fR.
The most useful are \fBGDK_ACTION_COPY\fR and \fBGDK_ACTION_MOVE\fR.
If you are for example passing around strings or other data, you will
most likely use \fBGDK_ACTION_COPY\fR only.
.P
Then you need to set up and bind the drop handler. The drop handler
should have the following prototype:
.VERBON 16
void  
target_drag_data_received  (GtkWidget          *widget,
			    GdkDragContext     *context,
			    gint                x,
			    gint                y,
			    GtkSelectionData   *data,
			    guint               info,
			    guint               time);
.VERBOFF
The data you have is in the structure \fRGtkSelectionData\fB, in the
\fBdata\fR field. That's all you need to do for normal DND.
Here's and example:
.VERBON 16
static void  
target_drag_data_received  (GtkWidget          *widget,
			    GdkDragContext     *context,
			    gint                x,
			    gint                y,
			    GtkSelectionData   *data,
			    guint               info,
			    guint               time)
{
	g_print("Got: %s\\n",data->data);
}
\&...
static GtkTargetEntry target_table[] = {
	{ "text/plain", 0, 0 }
}
\&...
gtk_drag_dest_set (widget,
		   GTK_DEST_DEFAULT_ALL,
		   target_table, 1,
		   GDK_ACTION_COPY);
gtk_signal_connect (GTK_OBJECT (widget), "drag_data_received",
		    GTK_SIGNAL_FUNC (target_drag_data_received),
		    NULL);
.VERBOFF
.P
For more information about drag and drop, you should see \fBGTK+\fR
documentation at \fBwww.gtk.org\fR.

.H 3 "Allowing Drags"
.P
Now let's look at the source side of DND. You set up the \fBGtkTargetEntry\fR
array, in the same manner as above. Then instead of the \fBflags\fR argument
you substitute a mask for the start mouse button of the drag. This could be
\fBGDK_BUTTON1_MASK | GDK_BUTTON3_MASK\fR for 1st and 3rd mouse buttons. Then
you need to bind the \fBdrag_data_get\fR signal that will send the data for
the drag on it's way, and \fBdrag_data_delete\fR if the action is
\fBGDK_ACTION_MOVE\fR, to delete the data since the move was successful.
Here's a simple example that will work with the above code snippet for drop:
.VERBON 16
static void  
source_drag_data_get  (GtkWidget          *widget,
		       GdkDragContext     *context,
		       GtkSelectionData   *selection_data,
		       guint               info,
		       guint               time,
		       gpointer            data)
{
	char string[] = "Some String!"
	gtk_selection_data_set (selection_data,
				selection_data->target,
				8, string, sizeof(string));
}
\&...
static GtkTargetEntry target_table[] = {
	{ "text/plain", 0, 0 }
}
\&...
gtk_drag_source_set (widget,
		     GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
		     target_table, 1,
		     GDK_ACTION_COPY);
gtk_signal_connect (GTK_OBJECT (widget), "drag_data_get",
		    GTK_SIGNAL_FUNC (source_drag_data_get),
		    NULL);
.VERBOFF
The \fRgtk_selection_data_set\fR function copies the data into the selection
data, which is used for the transfer.

.H 1 "Conclusion"
.H 2 "Getting Further Help"
.P
One of the best ways to get help with programming in gnome is probably
to first read the available documentation at \fBwww.gnome.org\fR. You
should also subscribe to the \fBgnome-list@gnome.org\fR, to subscribe,
send a message with \fBsubscribe\fR in the subject line to
\fBgnome-list-request@gnome.org\fR. To reduce the traffic on the
list you should first consult the documentation before asking a question.
Also look at \fBhttp://www.gnome.org/mailing-lists/\fR for a list of
all GNOME relevant mailing lists, including the \fBGTK+\fR list.
.P
However I still consider the header files for the libraries most helpful.
This is mostly as there isn't yet as much documentation out there as there
should be, but also the header files will always contain all of the
definitions and they will be up to date with the current version,
which a manual might not be. Most GTK+ and GNOME function names are very
descriptive and it's easy to figure out what they do. I use the header files
only. It's much easier to just look at the function prototype and figure out
what it does, then to hunt around in a reference manual. Then again you
usually have to know what header file to look at, which is not all
that hard, given that the header files are named by the objects or modules
they represent. For example the header file for \fBgnome-config\fR is
\fBlibgnome/gnome-config.h\fR. The header file for \fBGnomeCanvas\fR
is \fBlibgnomeui/gnome-canvas.h\fR.

.H 2 "Future GNOME Library Developments"
.P
This tutorial covers programming with version 1.0 of the gnome libraries.
But of course there is life after 1.0. There are many things still planned
for the libraries. Here's a short list of things that will be worked on.
.BL
.LI
More common dialogs, such as a native gnome file picker dialog.
.LI
More corba integration of the entire desktop. This is including much
more corba support from the core libraries.
.LI
Better canvas, including better alpha channel support and printing directly
from the canvas.
.LI
Rewrite of the configuration setup.
.LI
Much much more! ... Stuff we haven't even thought of yet!
.LE

.TC 1 1 3
